@@ -391,20 +391,92 @@ def gogogo_controls(
         return controls, params
 
 
-def prep_scalar(arg, name=None):
-    if isinstance(arg, tuple):
-        if isinstance(arg[0], Controls):
-            # index controls. e.g. ctrls['size']
+def _gen_f(key):
+    def f(*args, **kwargs):
+        return kwargs[key]
 
-            def f(*args, **kwargs):
-                return kwargs[arg[1][0]]
+    return f
 
-            return f, arg, None
-        elif name is not None:
-            # name will be set by calling function if from ipyplot
-            # this case is if given an abbreviation e.g.: `s = (0, 10)`
-            def f(*args, **kwargs):
-                return kwargs[name]
 
-            return f, None, (name, arg)
-    return arg, None, None
+def _gen_param_excluder(added_kwargs):
+    """
+    Pass through all the original keys, but exclude any kwargs that we added
+    manually through prep_scalar
+
+    Parameters
+    ----------
+    added_kwargs : list of str
+
+    Returns
+    -------
+    excluder : callable
+    """
+
+    def excluder(params, except_=None):
+        """
+        Parameters
+        ----------
+        params : dict
+        except : str
+        """
+        return {k: v for k, v in params.items() if k not in added_kwargs or k == except_}
+
+    return excluder
+
+
+def prep_scalars(kwarg_dict, **kwargs):
+    """
+    Process potentially scalar arguments. This allows for passing in
+    slider shorthands for these arguments, and for passing indexed controls objects for them.
+
+    Parameters
+    ----------
+    kwarg_dict : dict
+        The kwargs passed to the calling functions
+    kwargs :
+        The arguments to process
+
+    Returns
+    -------
+    funcs : dict
+        The processed arguments. Re-assign the args as `s = funcs['s']` to
+        make use of any autogenerated functions to connect callbacks to these
+        scalar values.
+    extra_ctrls : list of Controls
+        Any extra controls objects that need to be used. Useful for `gogogo_controls`
+    param_excluder : function
+        A function to exclude any scalar kwargs we added from params. Use this in the
+        update functions to avoid calling user provided functions with the kwargs that
+        we added here.
+    """
+    extra_ctrls = []
+    added_kwargs = []
+
+    for name, arg in kwargs.items():
+        if isinstance(arg, tuple):
+            if isinstance(arg[0], Controls):
+                # index controls. e.g. ctrls['size']
+                # arg looks like (controls, ['size'])
+
+                # factory function to avoid scoping shenanigans
+
+                extra_ctrls.append(arg)
+                kwargs[name] = _gen_f(arg[1][0])
+            elif name is not None:
+                # name will be set by calling function if from ipyplot
+                # this case is if given an abbreviation e.g.: `s = (0, 10)`
+
+                kwargs[name] = _gen_f(name)
+
+                # Modify the calling functions kwargs in place to add the arg
+                kwarg_dict[name] = arg
+                added_kwargs.append(name)
+
+    if len(added_kwargs) == 0:
+        # shortcircuit options
+        def param_excluder(params, except_=None):
+            return params
+
+    else:
+        param_excluder = _gen_param_excluder(added_kwargs)
+    return kwargs, extra_ctrls, param_excluder
